@@ -2,9 +2,9 @@ package Crypt::AuthEnc;
 
 use strict;
 use warnings;
-our $VERSION = '0.056';
+our $VERSION = '0.067';
 
-sub CLONE_SKIP { 1 } # prevent cloning
+### not used
 
 1;
 
