@@ -97,6 +97,9 @@ int register_all_ciphers(void)
 #endif
 #ifdef LTC_SERPENT
    REGISTER_CIPHER(&serpent_desc);
+#endif
+#ifdef LTC_TEA
+   REGISTER_CIPHER(&tea_desc);
 #endif
    return CRYPT_OK;
 }
