@@ -20,6 +20,8 @@ static const oid_table_entry pka_oids[] = {
                                               { PKA_DSA,       "1.2.840.10040.4.1" },
                                               { PKA_EC,        "1.2.840.10045.2.1" },
                                               { PKA_EC_PRIMEF, "1.2.840.10045.1.1" },
+                                              { PKA_X25519,    "1.3.101.110" },
+                                              { PKA_ED25519,   "1.3.101.112" },
 };
 
 /*
